//
// connection.cpp
// ~~~~~~~~~~~~~~
//
//

#include "connection.hpp"
#include <vector>
#include <boost/bind.hpp>
#include "connection_manager.hpp"
#include "request_handler.hpp"

namespace mpre {
  namespace network {
    
    connection::connection(boost::asio::io_service& io_service,
			   connection_manager& manager, request_handler& handler)
      : socket_(io_service),
	connection_manager_(manager),
	request_handler_(handler)
    {}

    boost::asio::ip::tcp::socket& connection::socket() {
      return socket_;
    }

    void connection::start(){
      socket_.async_read_some(boost::asio::buffer(buffer_, max_length),
			      boost::bind(&connection::handle_read, shared_from_this(),
					  boost::asio::placeholders::error,
					  boost::asio::placeholders::bytes_transferred));
    }

    void connection::stop() {
      socket_.close();
    }

    void connection::handle_read(const boost::system::error_code& e,
				 std::size_t bytes_transferred) {
      if (!e) {

	// FIXME! this is just a demo for echo server
	
	boost::asio::async_write(socket_, 
				 boost::asio::buffer(buffer_, bytes_transferred),
				 boost::bind(&connection::handle_write, shared_from_this(),
					     boost::asio::placeholders::error));
	
      }  
      // This is deferred when the keepalive check will discover it
      //
      ///else if (e != boost::asio::error::operation_aborted) {
      //
      //connection_manager_.stop(shared_from_this());
      //}
    }

    void connection::handle_write(const boost::system::error_code& e) {
      
      // On write, call read to continue echo

      if (!e) {
	socket_.async_read_some(boost::asio::buffer(buffer_, max_length),
				boost::bind(&connection::handle_read, this,
					    boost::asio::placeholders::error,
					    boost::asio::placeholders::bytes_transferred));
      } 
      //
      // deferred to next keep-alive check 
      //
      //else {
      //	delete this;
      //}
      //      if (!e) {
      // Initiate graceful connection closure.
      //	boost::system::error_code ignored_ec;
      // socket_.shutdown(boost::asio::ip::tcp::socket::shutdown_both, ignored_ec);
      // }
      //
      // if (e != boost::asio::error::operation_aborted) {
      // connection_manager_.stop(shared_from_this());
      //}
    }


    // The standard mechanism for checking if a socket is alive 
    // is to read some data from it
    void connection::check_keepalive(){
      
      std::cerr << "Calling socket checker";

	socket_.async_read_some(boost::asio::buffer(buffer_, max_length),
				boost::bind(&connection::handle_read_test, shared_from_this(),
					    boost::asio::placeholders::error,
					    boost::asio::placeholders::bytes_transferred));
    }

    //
    // If this empty op fails, then there is a problem on socket ( => peer dead! )
    //
    void connection::handle_read_test(const boost::system::error_code& e,
				 std::size_t bytes_transferred){

      if ((e) && (e != boost::asio::error::operation_aborted)) {
	
	connection_manager_.stop(shared_from_this());
	
	std::cerr << "A peer is dead\n";
      }
    }
    


  } // namespace network
} // namespace mpre
