//
// connection_manager.hpp
// ~~~~~~~~~~~~~~~~~~~~~~
//

#ifndef MPRE_CONNECTION_MANAGER_HPP
#define MPRE_CONNECTION_MANAGER_HPP

#include <set>
#include <boost/noncopyable.hpp>
#include "connection.hpp"

namespace mpre {
  namespace network {

    /// Manages open connections so that they may be cleanly stopped when the server
    /// needs to shut down and tested for keep alive
    class connection_manager
      : private boost::noncopyable
    {
    public:
      /// Add the specified connection to the manager and start it.
      void start(connection_ptr c);
      
      /// Stop the specified connection.
      void stop(connection_ptr c);
      
      /// Stop all connections.
      void stop_all();

      // check one connection for keepalive
      void check_keepalive(connection_ptr c);

      // Check all the connection
      void check_all_keepalive();

    private:
      /// The managed connections.
      std::set<connection_ptr> connections_;
    };

  } // namespace network
} // namespace mpre

#endif // MPRE_CONNECTION_MANAGER_HPP


