//
// server.hpp
// ~~~~~~~~~~
//
//

#ifndef MPRE_SERVER_HPP
#define HTTP_SERVER_HPP

#include <boost/asio.hpp>
#include <string>
#include <boost/noncopyable.hpp>

#include "connection.hpp"
#include "connection_manager.hpp"
#include "request_handler.hpp"

namespace mpre {
  namespace network {

    /// The top-level class of the MPRE server.
    class server
      : private boost::noncopyable
    {
    public:
      /// Construct the server to listen on the specified TCP address and port
      explicit server(const std::string& address, const std::string& port);

      
      /// Run the server's io_service loop.
      void run();
      
      /// Stop the server.
      void stop();

      // Set the timeout for keepalive
      void set_timeout_keepalive(unsigned short seconds);

      // handle for timeout -- call the connection_manager to check 
      // keep-alive on all the active sockets
      void handle_ontimeout();

      
    private:
      /// Handle completion of an asynchronous accept operation.
      void handle_accept(const boost::system::error_code& e);
      
      /// Handle a request to stop the server.
      void handle_stop();
      
      /// The io_service used to perform asynchronous operations.
      boost::asio::io_service io_service_;

      // The timer used to check the socket keep alive
      boost::asio::deadline_timer t_;

      // timeout in seconds -- 0 means nothing
      unsigned short sectimeout_;
      
      /// Acceptor used to listen for incoming connections.
      boost::asio::ip::tcp::acceptor acceptor_;
      
      /// The connection manager which owns all live connections.
      connection_manager connection_manager_;

      /// The next connection to be accepted.
      connection_ptr new_connection_;

      /// The handler for all incoming requests.
      request_handler request_handler_;
    };
    
  } // namespace mpre
} // namespace network

#endif // MPRE_SERVER_HPP
